// TroutGun.java
//
// ICS 22 / CSE 22 Fall 2012
// Code Example
//
// TroutGuns are kinds of weapons, so we extend the TroutGun class from
// Weapon.  We want to be able to create TroutGun objects -- they're an
// actual kind of weapon that we want to have in our game -- so we want
// this class not to be abstract, which obligates us to write a
// canFireWithAmmunition() method to fill in the "hole" left in the
// Weapon class.
//
// We'll say that TroutGuns behave in the following special ways:
//
// * Because trout aren't a very hearty kind of ammunition, trout guns
//   have the unfortunate habit of jamming.  On the average, trout guns
//   jam up one out of every four times you try to fire them.
// * Once a trout gun is jammed, it can't be fired successfully again
//   until it's cleaned.
//
// This behavior is implemented in the class below.  We use random number
// generation to handle the "one out of every four times" part, while using
// a boolean field to store, at any given time, whether the trout gun is
// jammed.  Finally, we add a clean() method, unique to TroutGun, which
// cleans the gun and makes it ready to be fired again.

import java.util.Random;


public class TroutGun extends Weapon
{
	private Random random;
	private boolean isJammed;


	public TroutGun(int ammunitionCount)
	{
		super(ammunitionCount);
		random = new Random();
		isJammed = false;
	}
	
	
	public void clean()
	{
		isJammed = false;
	}


	public boolean canFireWithAmmunition()
	{
		return !isJammed;
	}


	// Firing a TroutGun is different from firing other weapons, because of
	// the possibility of it jamming.  This version of fire() is said to
	// "override" the version inherited from Weapon.  When you ask a TroutGun
	// to fire, this method will be called instead of the version from Weapon.
	//
	// Note, too, that most of what's being done here is being done by
	// the Weapon version of fire() -- super.fire() is a call to the fire()
	// method in the superclass, i.e., Weapon's version of fire().  After
	// that's done, we separately check whether the weapon is jammed.
	// (random.nextInt(4) will return 0, on the average, 25% of the time,
	// so 25% of the time, the gun will jam.)
	public void fire()
	throws CannotFireException
	{
		super.fire();

		if (random.nextInt(4) == 0)
		{
			isJammed = true;
			throw new CannotFireException();
		}
	}
}
